function [lkh means1 params1]=hsmmEstimateConds3(factors,conditions,means,params,thresh)
% 
% Author: Shun-Zheng Yu
% Available: http://sist.sysu.edu.cn/~syu/Publications/hsmm_new.m

% 
% Ref: Practical Implementation of an Efficient Forward-Backward Algorithm for an Explicit Duration Hidden Markov Model
% by Shun-Zheng Yu and H. Kobayashi
% IEEE Transactions on Signal Processing, Vol. 54, No. 5, MAY 2006, pp. 1947-1951 
% 
%  This program is free software; you can redistribute it and/or
%  modify it under the terms of the GNU General Public License
%  as published by the Free Software Foundation; either version
%  2 of the License, or (at your option) any later version.
%  http://www.gnu.org/licenses/gpl.txt
%
%++++++++ Markov Model +++++++++++

maxlength = max(factors(:,1));

%Jelmers test base
%disp(maxlength)
%disp(size(factors))
%disp(size(conditions))
%disp(size(means))
%disp(size(params))
%disp(thresh)

%Jerryrigging
nscans=size(factors,1);
nvals = size(factors,2)-1;
nstates = size(means,2);
M = nstates+1;
A = cat(1,cat(2,zeros(nstates,1),eye(nstates)),cat(2,1,zeros(1,nstates)));
nconds=max(conditions);
D=maxlength;                 %The maximum duration of a trial
Ah = A; %Remember the transition matrix before skips
ALPHA=zeros(M,D);
Pest=zeros(M,D,nconds);
Aest=zeros(M,M);
PAI=zeros(M,1);
if ndims(params) == 2
    params = repmat(params,[1,1,nconds]);
end
PAI(M)=1; 
%creates a start state vector starting in the rest state
% The two lines above create a sequence of events with sinngle rest scans
% forced
% beteen the trials.  It is a matrix B where the columns represent the
% original scans plus the inserted rest scans and the rows represent the
% states with the last state being the rest state.  The hsmm is forced to
% go into the rest state on rest scans because the probability all other
% states is set to zero on this scan and the rest state has probability 1.
% On all other scans the rest state has probability zero and the other
% states have the probability of that scan.  These probabilities sum to 1
% but basetot reflects an additional log likelikelihood to be added in.
%----------------------------------------------------
P=zeros(M,maxlength,nconds);
for i = 1:M-1
    for j = 1:nconds
        P(i,:,j)=gamma21(params(i,1,j),params(i,2,j),maxlength)';
    end
end
%disp(P)
%disp(M)
P(M,1,:) = 1;
if thresh == 0
    bound = 1;
else
    bound = 200;
end
E2=zeros(M-1,nscans);
for ir=1:bound
    %throughout the loop  about-to-be-calculated basetot and lkh represent
    %the value of the new parameters.   basetot1 and lkh1 represent the
    %values of the old parameters
    if ir > 1
        lkh1=lkh;
        basetot1=basetot;
    else
        lkh1=-Inf;
        basetot1=-Inf;
    end
    %fprintf('basetot: %d\n', basetot1);
    basetot = 0;
    lkh = 0;
    for iz = 1:nconds
        values=factors(conditions==iz,:);
        trials=values(:,1);
        values=values(:,2:end);
        ntrials=sum(trials==1);
        %fprintf('mean values: %f\n', mean(mean(values)));
        %fprintf('mean means: %f\n', mean(mean(means)));

        [B basetotx] = constructProbs(values,means,trials);
        %fprintf('basetotx: %d\n', basetotx);

        basetot=basetot+basetotx;
       % fprintf('basetot2: %d\n', basetot);
        B = B';
        ends = find(B(M,:)==1);
        T=size(B,2);    %The total number of scans
        bmx=zeros(M,T);
        S=zeros(M,T);
        E=zeros(M,T);
        Best=zeros(M,T);
        Qest=zeros(T,1);
        
       % disp(iz)
        A = skips(Ah,params(:,:,iz),maxlength);
       % disp(A)
        
%++++++++++++++++++     Forward     +++++++++++++++++
        %---------------    Initialization    ---------------
        ALPHA(:)=0; ALPHA=repmat(PAI,1,D).*P(:,:,iz);		%Equation (13)
        r=(B(:,1)'*sum(ALPHA,2));			%Equation (3)
        bmx(:,1)=B(:,1)./r;			%Equation (2)
        E(:)=0; E(:,1)=bmx(:,1).*ALPHA(:,1);	%Equation (5)
        
       
        
        S(:)=0; S(:,1)=A'*E(:,1);			%Equation (6)
        lkh=lkh+log(r);
        %disp()
%---------------    Induction    ---------------
        %fprintf('ALPHA: %f\n', mean(mean(ALPHA)));
        %fprintf('ALPHA3: %f S: %f D %f P %f bmx %f iz %f T %f\n',...
         %   mean(mean(ALPHA)), mean(mean(S)), mean(D), mean(mean(mean(P))), mean(mean(bmx)), iz, T);
        %disp(size(B))
        %disp(S)
        for t=2:T
           % disp(S(:,t-1))  

            ALPHA=[repmat(S(:,t-1),1,D-1).*P(:,1:D-1,iz)+repmat(bmx(:,t-1),1,D-1).*ALPHA(:,2:D),S(:,t-1).*P(:,D,iz)]	;	%Equation (12)
            %fprintf('\tfor ALPHA size: %f\n', ));
           % disp(sum(ALPHA,2))
           % disp(B(:,t))
            r=(B(:,t)'*sum(ALPHA,2));		%Equation (3)
            %fprintf('r: %f\n', mean(mean(r)));
            bmx(:,t)=B(:,t)./r;			%Equation (2)
            E(:,t)=bmx(:,t).*ALPHA(:,1);		%Equation (5)
            S(:,t)=A'*E(:,t);
            %Equation (6)
            lkh=lkh+log(r);
            %fprintf('\tfor ALPHA3: %f S: %f D %f P %f bmx %f iz %f T %f\n',...
            %mean(mean(ALPHA)), mean(mean(S)), mean(D), mean(mean(mean(P))), mean(mean(bmx)), iz, T);
       
        end
       %  fprintf('ALPHA2: %f\n', mean(mean(ALPHA)));
        
%++++++++ Backward and Parameter Restimation ++++++++
       if thresh>0            % re-estimate parameters
    %---------------    Initialization    ---------------

            Aest(:)=0;  Aest=E(:,T)*ones(1,M);  %Since T_{T|T}(m,n) = E_{T}(m) a_{mn}
            Pest(:,:,iz)=0;  
            GAMMA=bmx(:,T).*sum(ALPHA,2);
            %fprintf('ALPHA: %f\n', mean(ALPHA));
            Best(:)=0;
            Best(:,T)=GAMMA;
            [X,Qest(T)]=max(GAMMA);

            BETA=repmat(bmx(:,T),1,D);				%Equation (7)
            Ex=sum(P(:,:,iz).*BETA,2);					%Equation (8)
            Sx=A*Ex;						%Equation (9)

    %---------------    Induction    ---------------
            for t=(T-1):-1:1
                %% for estimate of A
                Aest=Aest+E(:,t)*Ex';
                %% for estimate of P
                Pest(:,:,iz)=Pest(:,:,iz)+repmat(S(:,t),1,D).*BETA;
                %% for estimate of state at time t
                GAMMA=GAMMA+E(:,t).*Sx-S(:,t).*Ex;
                
                GAMMA(GAMMA<0)=0;           % eleminate errors due to inaccurace of the computation.
                [X,Qest(t)]=max(GAMMA);
                %% for estimate of B
                Best(:,t)=Best(:,t)+GAMMA;

                BETA=repmat(bmx(:,t),1,D).*[Sx,BETA(:,1:D-1)];	%Equation (14)
                Ex=sum(P(:,:,iz).*BETA,2);					%Equation (8)
                Sx=A*Ex;						%Equation (9)
            end

            Pest(:,:,iz)=Pest(:,:,iz)+repmat(PAI,1,D).*BETA;    %Since D_{1|T}(m,d) = \PAI(m) P_{m}(d) \Beta_{1}(m,d)
            Pest(:,:,iz)=Pest(:,:,iz).*P(:,:,iz);   
            P(:,:,iz)=zeros(M,maxlength);
            %fprintf('Best: %f\n', Best);
            E1 = Best;
            E1(:,ends)=[];
            E1(M,:)=[];
            E2(:,conditions==iz)=E1;
            for i = 1:M-1
                Pest1=cat(2,max(0,ntrials -sum(Pest(i,:,iz))),Pest(i,:,iz));
                %it is possible that there are more visits hence the bound
                %max(0,counts(i) -sum(Pest(i,:)))
                if sum(Pest(i,:,iz)>1)>2    %only re-estimate if there is enough distinct points 
                        temp = gamfit(cat(2,.5,[2:2:2*maxlength]),[],[],Pest1);
                        if prod(temp,2) > 1  %constrain durations to be above 1 second
                            params(i,:,iz) = temp;
                        end  
                end
                P(i,:,iz)=gamma21(params(i,1,iz),params(i,2,iz),maxlength)';
            end
            P(M,1,iz) = 1;
        end
    end
    for i = 1:M-1
        
       % fprintf('\n\nnvals: %f mean E2: %f\n\n', nvals, mean(mean(E2)));

        weights =repmat(E2(i,:),nvals,1)';
        
        %fprintf('\n\nmean weights: %f\n\n', mean(mean(weights)));
        means(:,i)=sum(factors(:,2:end).*weights)./sum(weights);
        %fprintf('\n\nmean means 2: %f\n\n', mean(mean(means)));
    end
  %++++++++ To check if the likelihood is increased ++++++++
    if ir > 1 && (basetot+lkh-basetot1-lkh1)<thresh  
        break
    elseif thresh == 0
        params1=params;
        means1=means;
        break
    else
        params1=params;
        means1=means;
    end
end
lkh=max(lkh+basetot,lkh1+basetot);
%disp(basetot)
%disp(lkh)